# Generated by roxygen2: do not edit by hand

export("%>%")
export(H2cal)
export(design_augmented)
export(design_noreps)
export(design_repblock)
export(design_split)
export(design_split_rcbd)
export(footnotes)
export(include_pdf)
export(include_table)
export(mean_comparison)
export(metamorphosis)
export(plot_augmented_design)
export(plot_diagnostic)
export(plot_raw)
export(plot_smr)
export(plot_split_rcbd_design)
export(plot_standard_design)
export(remove_outliers)
export(rticle)
export(split_folder)
export(tarpuy)
export(tarpuy_design)
export(tarpuy_plex)
export(tarpuy_plotdesign)
export(tarpuy_traits)
export(web_table)
export(yupana)
export(yupana_analysis)
export(yupana_export)
export(yupana_import)
export(yupana_mvr)
export(yupana_reshape)
import(dplyr)
import(ggplot2)
importFrom(DT,datatable)
importFrom(agricolae,
  HSD.test,
  SNK.test,
  duncan.test
)
importFrom(dplyr,
  "%>%",
  across,
  all_of,
  filter,
  mutate,
  mutate_all,
  mutate_at,
  pull,
  rename,
  rename_at,
  select,
  starts_with,
  sym,
  vars
)
importFrom(emmeans,emmeans)
importFrom(ggplot2,
  aes,
  geom_point,
  ggplot,
  theme_minimal
)
importFrom(grDevices,
  colorRampPalette,
  colors,
  gray.colors
)
importFrom(graphics,
  abline,
  hist,
  par
)
importFrom(lme4,
  VarCorr,
  lmer,
  ranef
)
importFrom(purrr,
  as_vector,
  discard,
  pluck
)
importFrom(shiny,
  dialogViewer,
  runApp,
  runGadget
)
importFrom(stats,
  anova,
  aov,
  as.formula,
  density,
  fitted,
  lm,
  median,
  p.adjust,
  pnorm,
  qqline,
  qqnorm,
  resid,
  residuals,
  sd,
  var,
  vcov
)
importFrom(stringr,
  str_count,
  str_detect,
  str_extract,
  str_extract_all,
  str_replace,
  str_to_upper,
  word
)
importFrom(tibble,
  as_tibble,
  deframe,
  enframe,
  rownames_to_column,
  tibble,
  tribble
)
importFrom(tidyr,
  drop_na,
  pivot_longer,
  pivot_wider,
  separate
)
importFrom(utils,
  packageVersion,
  read.csv,
  tail
)
