---
title: "SciHub"
format:
  html:
    toc: true
    toc-expand: true
    toc-depth: 4
    toc-location: left
    number-sections: true
    self-contained: true
    code-fold: false
editor_options: 
  chunk_output_type: console
execute: 
  warning: false
  message: false
  echo: true
---

# Information

**Manuscript Title**

Author List

---

# Manuscript

> https://docs.google.com/document/d/1B5wtKmWap6MHYjVJqlYy7HMXHWQDoprlz-om2nn777s/edit?usp=sharing

# Database

> https://docs.google.com/spreadsheets/d/1u6xgTT8jDFkHSMkK1iRrNVR-1iOINQidbEiaDCu2Nqw/edit?usp=sharing

# Presentation

> Google Slides link

---

# *SciHub*: research pipeline repository

```{}
project-root/
  ├── analysis.qmd          # Script with data analysis
  ├── article.qmd           # Script for formatting the article
  ├── article.zip           # Zipped file in Markdown from Articul8
  ├── design/               # Experimental design and setup
  │   ├── design                # Information of experimental setup and planning
  │   └── labels.pdf            # Labels script from experiments
  ├── files/                # Supporting files for the project
  │   ├── images/               # Raw and processed images
  │   ├── zotero/               # Zotero library (.rds)
  │   └── template.docx         # Journal article template  
  ├── manuscript/           # Core outputs of the article
  │   ├── Figures               # Figures for the manuscript (.jpg & .eps)
  │   ├── ESM                   # Electronic Supplementary Material (ESM)
  │   └── draft.docx            # Manuscript text versions
  └── README.qmd            # Article title with authors:
                                # Manuscript link (Google Docs)
                                # Database link (Google Sheets)
                                # Presentation link (Google Slides)
```

