---
title: Tarpuy
description: > 
  Tarpuy: field-book experimental plans.
author: "Flavio Lozano-Isla"
vignette: >
  %\VignetteIndexEntry{tarpuy}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
knitr:
  opts_chunk: 
    collapse: true
    comment: '#>'
---

```{r setup, include=FALSE}
source("https://raw.githubusercontent.com/Flavjack/inti/master/pkgdown/favicon/docs.r")
```

Tarpuy es una herramienta que permite la construcción de planes experimentales de forma interactiva.

Tarpuy permite:

- Generación de planes experimentales
- Construcción de experimentos basados en diseños experimentales
- Libros de campo para las evaluación de experimentos 
- Incluir lista de variables en los libros de campo
- Croquis basados en los diseños experimentales
- Permite adicionar: cronograma, presupuesto y diario

```{=html}
<div id=footer style="width:100%; margin:auto;">

<div style="display:inline-block; width:49%">
<p style="text-align:center">
<a target="_blank" href="https://www.youtube.com/playlist?list=PLSQMdOu57lj8XTyH5KUN9h-VL5TAEsaBC">
<img src="https://flavjack.github.io/inti/img/youtube.png" style="height:65px" title="demo"></a>
<span style="display:block;"><small>demo</small></span>
</p></div>

<div style="display:inline-block; width:49%">
<p style="text-align:center">
<a target="_blank" href="https://inkaverse.shinyapps.io/tarpuy/"><img src="https://flavjack.github.io/inti/img/tarpuy.png" style="height:80px" title="tarpuy"></a>
<span style="display:block;"><small>Tarpuy</small></span>
</p></div>

</div>
```

# Módulos

```{r}
url <- "https://docs.google.com/spreadsheets/d/1nLP-dH9-dsVMXgsU7Z02EbgWvMIBKG5IYVRHAJCN-Z0/edit#gid=73943593"
url %>% 
  gsheet2tbl() %>% 
  include_table(caption = "Módulos de la aplicación Tarpuy")
```
