---
title: Apps
description: > 
  inti: interactive apps.
author: "Flavio Lozano-Isla"
vignette: >
  %\VignetteIndexEntry{Apps}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{quarto::html}
knitr:
  opts_chunk: 
    collapse: true
    comment: '#>'
    echo: false
    warning: false
    message: false
editor_options: 
  chunk_output_type: console
---

The package `inti` has been developed to provide tools used in plant science and experimental designs. The package also include shiny apps for design experiments, interactive analysis and technical writing.

The `inti` package is inspired by the Inca mythology where the sun (`inti`) is considered the most significant deity. The web applications are based on this worldview.

The shiny apps are based in the functions developed in the package <https://inkaverse.com/reference/>.

## Install the apps locally

```{=html}
<p align="center"><iframe width="560" height="315" src="https://www.youtube.com/embed/sbpBD5CyUGM" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe></p>
```

In the case you need to change the email account o renew the credentials to access to the apps you can use `googlesheets4::gs4_token()`.

## Tarpuy

Ease way to deploy field-book experimental plans.

```{=html}
<div id=footer style="width:100%; margin:auto;">

<div style="display:inline-block; width:32%">
<p style="text-align:center">
<a target="_blank" href="https://www.youtube.com/playlist?list=PLSQMdOu57lj8XTyH5KUN9h-VL5TAEsaBC">
<img src="https://flavjack.github.io/inti/img/youtube.png" style="height:80px" title="demo"></a>
<span style="display:block;"><small>demo</small></span>
</p></div>

<div style="display:inline-block; width:32%">
<p style="text-align:center">
<a target="_blank" href="https://inkaverse.com/articles/tarpuy.html">
<img src="https://lozanoisla.com/img/tools.png" style="height:80px" title="options"></a>
<span style="display:block;"><small>options</small></span>
</p></div>

<div style="display:inline-block; width:32%">
<p style="text-align:center">
<a target="_blank" href="https://inkaverse.shinyapps.io/tarpuy/"><img src="https://flavjack.github.io/inti/img/tarpuy.png" style="height:80px" title="tarpuy"></a>
<span style="display:block;"><small>Tarpuy</small></span>
</p></div>

</div>
```

## Yupana

Data analysis and graphics for experimental designs.

```{=html}
<div id=footer style="width:100%; margin:auto;">

<div style="display:inline-block; width:32%">
<p style="text-align:center">
<a target="_blank" href="https://www.youtube.com/playlist?list=PLSQMdOu57lj9sTx5Dbff9O0g6KCU4pwCQ">
<img src="https://flavjack.github.io/inti/img/youtube.png" style="height:80px" title="demo"></a>
<span style="display:block;"><small>demo</small></span>
</p></div>

<div style="display:inline-block; width:32%">
<p style="text-align:center">
<a target="_blank" href="https://inkaverse.com/articles/yupana.html">
<img src="https://lozanoisla.com/img/tools.png" style="height:80px" title="options"></a>
<span style="display:block;"><small>options</small></span>
</p></div>

<div style="display:inline-block; width:32%">
<p style="text-align:center">
<a target="_blank" href="https://inkaverse.shinyapps.io/yupana/"><img src="https://flavjack.github.io/inti/img/yupana.png" style="height:80px" title="yupana"></a>
<span style="display:block;"><small>Yupana</small></span>
</p></div>

</div>
```

## Huito

An open-source R package to deploys flexible and reproducible labels using layers. 

***

```{=html}
<div id=footer style="width:100%; margin:auto;">

<div style="display:inline-block; width:49%">
<p style="text-align:center">
<a target="_blank" href="https://cran.r-project.org/package=huito"><img src="https://huito.inkaverse.com/logo.png" style="height:80px" title="R cran"></a> 
<span style="display:block;"><small>Huito</small></span>
</p></div>

<div style="display:inline-block; width:49%">
<p style="text-align:center">
<a target="_blank" href="https://huito.inkaverse.com/"><img src="https://pkgdown.r-lib.org/reference/figures/logo.png" style="height:80px" title="Huito"></a> 
<span style="display:block;"><small>Project</small></span>
</p></div>

</div>
```

## GerminaR + GerminaQuant

GerminaR is the first platform base in open source package to calculate and graphic the germination indices in R. GerminaR include a web application called "GerminQuant for R" for non programming users.

***

```{=html}
<div id=footer style="width:100%; margin:auto;">

<div style="display:inline-block; width:24%">
<p style="text-align:center">
<a target="_blank" href="https://CRAN.R-project.org/package=GerminaR"><img src="https://flavjack.github.io/GerminaR/reference/figures/logo.png" style="height:80px" title="R cran" alt="GerminaR"></a> 
<span style="display:block;"><small>GerminaR</small></span>
</p></div>

<div style="display:inline-block; width:24%">
<p style="text-align:center">
<a target="_blank" href="https://www.youtube.com/playlist?list=PLSQMdOu57lj9QUoA6JLuv0nBTYacuyrbg"><img src="https://lozanoisla.com/img/youtube.png" style="height:80px" title="Demo" alt="Demo"></a> 
<span style="display:block;"><small>Demo</small></span>
</p></div>

<div style="display:inline-block; width:24%">
<p style="text-align:center">
<a target="_blank" href="https://inkaverse.shinyapps.io/germinaquant/"><img src="https://germinar.inkaverse.com/img/GerminaQuant.png" style="height:80px" title="GerminaQuant" alt="GerminaQuant for R"></a>
<span style="display:block;"><small>GerminaQuant</small></span>
</p></div>

<div style="display:inline-block; width:24%">
<p style="text-align:center">
<a target="_blank" href="https://germinar.inkaverse.com/"><img src="https://pkgdown.r-lib.org/reference/figures/logo.png" style="height:80px" title="GerminaR" alt="GerminaR"></a> 
<span style="display:block;"><small>Project</small></span>
</p></div>

</div>
```

### Citation

Lozano-Isla, Flavio; Benites-Alfaro, Omar Eduardo; Pompelli, Marcelo Francisco (2019). GerminaR: An R package for germination analysis with the interactive web application “GerminaQuant for R.” Ecological Research, 34(2), 339–346. <https://doi.org/10.1111/1440-1703.1275>

